      subroutine bwtkk(u1,n1,p1,u2,n2,p2,u,n,p,ip,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns the spinor array
c                  u=f*e*(gp*P+ + gm*P-)*(p+m)/den
c
c Input: u1 - spinor
c        p1,p2 - momenta,
c        n1,n2,n - numbers of polarization states,
c        g - a fermion-fermion-boson coupling,
c        mass, wdth - a mass width of u
c        ip=+1/-1 for u representing a particle/an antiparticle
c Output: u - fermion array,
c         p - its momentum.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 27 March, 2006; modified on 10 April, 2006; 27 April, 2006,
c                             16 February, 2007,
c                             6/13 May (name and interface changed, 
c                             used to be fvkk)
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: n1,n2,n,ip,i1,i2,i
      real(dp), dimension(0:3) :: p1,p2,p
      real(dp) :: mass,wdth
      complex(dp), parameter :: ocom=(0.d0,0.d0),icom=(0.d0,1.d0)
      complex(dp) :: g(2),u1(4,n1),u2(0:3,n2),u(4,n),
     &               mf,mf2,den,gp,gm,mgp,mgm,ubem(2),ubep(2)
      complex(dp), dimension(4) :: pp,pm


      mf2=cmplx(mass**2,-mass*wdth,dp)
      mf=sqrt(mf2)

      p=p1+p2

      pp=ip*(/cmplx(p(0)-p(3),0.d0,dp),   ! (1,1)
     &        cmplx(-p(1),-p(2),dp),      ! (2,1)
     &        cmplx(-p(1),p(2),dp),       ! (1,2)
     &        cmplx(p(0)+p(3),0.d0,dp) /) ! (2,2)

      pm=(/pp(4),-pp(2),-pp(3),pp(1)/)

      den=pp(1)*pp(4)-pp(3)*pp(2)-mf2

      gp=g(1)/den
      gm=g(2)/den
      mgp=mf*gp
      mgm=mf*gm

      i=0

      do i2=1,n2
       do i1=1,n1
        i=i+1

        ubem=(/u1(3,i1)*(u2(0,i2)+u2(3,i2))
     &        +u1(4,i1)*(u2(1,i2)+icom*u2(2,i2)),
     &         u1(3,i1)*(u2(1,i2)-icom*u2(2,i2))
     &        +u1(4,i1)*(u2(0,i2)-u2(3,i2))/)

        u(:,i)=(/mgm*ubem(1),
     &           mgm*ubem(2),
     &           gm*(ubem(1)*pp(1)+ubem(2)*pp(2)),
     &           gm*(ubem(1)*pp(3)+ubem(2)*pp(4))/)

        if (g(1) /= ocom) then
         ubep=(/u1(1,i1)*(u2(0,i2)-u2(3,i2))
     &         +u1(2,i1)*(-u2(1,i2)-icom*u2(2,i2)),
     &          u1(1,i1)*(-u2(1,i2)+icom*u2(2,i2))
     &         +u1(2,i1)*(u2(0,i2)+u2(3,i2))/)
         u(:,i)=u(:,i)+(/gp*(ubep(1)*pm(1)+ubep(2)*pm(2)),
     &                   gp*(ubep(1)*pm(3)+ubep(2)*pm(4)),
     &                   mgp*ubep(1),
     &                   mgp*ubep(2)/)
        end if
       end do
      end do

      end subroutine bwtkk
