      subroutine kinffmap(imom,ecm,m1,m2,rand,ct1,q,p1,p2,dlips)
c----------------------------------------------------------------------c
c Returns cartezian components of the final state four momenta and
c the phase space normalization for a 2 --> 2 process in the c.m.s. 
c with cosine of the polar angle generated with the didtribution
c ~1/[(a-ct)(b+ct)], where a=(2q(0)p1(0)-q^2)/(2 qq pp),
c b=(2q(0)p2(0)-q^2)/(2 qq pp).
c The factor (2*pi)^-2 is not included in dlips.
c
c Input: imom=1/(else) => momenta p1 and p2 are/(not) calculated
c        ecm - the c.m.s. energy,
c        m1,m2 - particle masses,
c        rand(i) - random variables, 0 < rand(i) < 1, i=1,2,
c        q - a four momentum in denominators (q-p1)^2-m1^2 and 
c            (q-p2)^2-m2^2
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 10 Dec., 2010; modified on 
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: imom
      real(dp) :: ecm,m1,m2,rand(2),ct1,q(0:3),p1(0:3),p2(0:3),
     &            pp,qq,qq2,dlips
      real(dp), parameter :: pi=3.141592653589793238d0, zero=0.d0,
     &                       dphi=2.d0*pi,cps=dphi/4.d0
      real(dp) :: ct,dct,pst,phi,delme,e1,e2,a,b,d,dr

      delme=m1+m2

      if (ecm < delme) then
       dlips=zero
      else
       delme=(m1-m2)*delme/ecm
       if (delme == 0.d0) then
        e1=0.5d0*ecm
        e2=e1
        pp=sqrt((e1-m1)*(e1+m1))
       else
        e1=0.5d0*(ecm+delme)
        e2=e1-delme
        if (e1 < m1 .or. e2 < m2) then 
         dlips=0.d0
         return
        end if
        if (delme > 0.d0) then
         pp=sqrt((e1-m1)*(e1+m1))
        else
         pp=sqrt((e2-m2)*(e2+m2))
        end if
       end if

       qq=sqrt(q(1)**2+q(2)**2+q(3)**2)
       qq2=(q(0)-qq)*(q(0)+qq)

       a=(2.d0*q(0)*e1-qq2)/(2.d0*qq*pp)
       b=(2.d0*q(0)*e2-qq2)/(2.d0*qq*pp)
       d=((a+1.d0)/(a-1.d0))*((b+1.d0)/(b-1.d0))
       if (imom == 1) then
        dr=(b-1.d0)/(a+1.d0)*d**rand(1)
        ct=(a*dr-b)/(1.d0+dr)
        if (ct < 1.d0) then
         pst=pp*sqrt((1.d0-ct)*(1.d0+ct))
        else
         pst=0.d0
        end if
        phi=dphi*rand(2)
        p1=(/e1,pst*cos(phi),pst*sin(phi),pp*ct/)
        p2=(/e2,-p1(1),-p1(2),-p1(3)/)
       else
        ct=ct1
       end if
       dct=(a-ct)*(b+ct)/(a+b)*log(d)
       dlips=cps*pp/ecm*dct

      end if

      end


