      subroutine calcdis(p,cross)
c----------------------------------------------------------------------c
c Calculates distributions
c
c Input:  p - particle momenta,
c         cross - corresponding cross section.
c Select the number 'ndist' of distributions to be calculated and
c adjust the maximum number of bins 'nbmax' in 'distribs.f'
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 31 October, 2008, modiffied on 3 March, 2013
c----------------------------------------------------------------------c
      use inprms
      use drivekk
      use distribs
      use qrkinds
      use prtinds

      implicit none

      integer :: i,j,k,imxup=0
      real(dp) :: cross,p(0:3,nepl),ptl,ppl,pb(0:3),pl(0:3),ph(0:3),
     &            pn(0:3),pt(0:3),pw(0:3),sqs,g,v(3)
      real :: rand
      character(80) ind
      save imxup

      if (idisn == 1) then
       idisn=0
       nbs=0
       xmin=0.d0
       xmax=0.d0
c Distrbution 1:
       desc(1)='Rapidity of the lepton'
       nbs(1)=20
       xmin(1)=-2.1d0
       xmax(1)=2.1d0
c Distrbution 2:
       desc(2)='pT of the lepton'
       nbs(2)=40
       xmin(2)=30.d0
       xmax(2)=500.d0
c Distrbution 3:
       desc(3)='Cosine of the lepton with respect to beam'
       nbs(3)=20
       xmin(3)=-1.d0
       xmax(3)=1.d0
c Distrbution 4:
       desc(4)='Energy of the lepton'
       nbs(4)=40
       xmin(4)=30.d0
       xmax(4)=500.d0
c Distrbution 5:
       desc(5)='Cosine of the angle between p_l and p_t in top rest'//
     &         ' frame' 
       nbs(5)=20
       xmin(5)=-1.d0
       xmax(5)=1.d0
c Distrbution 6:
       desc(6)='Cosine of the angle between p_l and -p_b in top rest'//
     &         ' frame'
       nbs(6)=20
       xmin(6)=-1.d0
       xmax(6)=1.d0
c Distrbution 7:
       desc(7)='Cosine of the angle between p_l and -p_b in W rest'//
     &         ' frame'
       nbs(7)=20
       xmin(7)=-1.d0
       xmax(7)=1.d0
c Distrbution 8:
       desc(8)='Cosine of the angle between p_h and p_t in LAB'//
     &         ' frame'
       nbs(8)=20
       xmin(8)=-1.d0
       xmax(8)=1.d0
c Distrbution 9:
       desc(9)='Cosine of the angle between p_l and p_h in LAB'//
     &         ' frame'
       nbs(9)=20
       xmin(9)=-1.d0
       xmax(9)=1.d0
c Distrbution 10:
       desc(10)='Invariant mass of two b-quarks'
       nbs(10)=50
       xmin(10)=0.d0
       xmax(10)=200.d0

       do j=1,ndist
        if (nbs(j) > 0) then
         xdi(0,j)=xmin(j)
         dx(:,j)=(xmax(j)-xmin(j))/nbs(j)
         do i=1,nbs(j)
          xdi(i,j)=xdi(i-1,j)+dx(i,j)
         end do
        end if
       end do
      end if

      xd(1)=0.5d0*log((p(0,ilept)+p(3,ilept))/(p(0,ilept)-p(3,ilept)))
      ptl=sqrt(p(1,ilept)**2+p(2,ilept)**2)
      xd(2)=ptl
      ppl=sqrt(ptl**2+p(3,ilept)**2)
      xd(3)=p(3,ilept)/ppl
      xd(4)=p(0,ilept)

      if (ib4 > 0) then
c Boost to the top rest frame:
       pb=p(:,ib4)
       pl=p(:,ilept)
       pn=p(:,ineut)
       pt=pb+pl+pn
       sqs=sqrt(pt(0)**2-pt(1)**2-pt(2)**2-pt(3)**2)
       v=pt(1:)/pt(0)
       g=pt(0)/sqs
       call boost(g,-v,pl)
       call boost(g,-v,pn)
       call boost(g,-v,pb)
       xd(5)=(pl(1)*pt(1)+pl(2)*pt(2)+pl(3)*pt(3))
     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
     &            /sqrt(pt(1)**2+pt(2)**2+pt(3)**2)
c Reverse b momentum:
       pb(1:)=-pb(1:)
       xd(6)=(pl(1)*pb(1)+pl(2)*pb(2)+pl(3)*pb(3))
     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
     &            /sqrt(pb(1)**2+pb(2)**2+pb(3)**2)
c Reverse b momentum and boost to the W rest frame:
       pw=pl+pn
       sqs=sqrt(pw(0)**2-pw(1)**2-pw(2)**2-pw(3)**2)
       v=pw(1:)/pw(0)
       g=pw(0)/sqs
       call boost(g,-v,pl)
       call boost(g,-v,pb)
       xd(7)=(pl(1)*pb(1)+pl(2)*pb(2)+pl(3)*pb(3))
     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
     &            /sqrt(pb(1)**2+pb(2)**2+pb(3)**2)
      else
       xd(5:7)=xmin(5:7)-10.d0
      end if


      if (ib1 > 0 .and. ib4 > 0) then
       ph=p(:,ib1)+p(:,ib2)
       xd(8)=(ph(1)*pt(1)+ph(2)*pt(2)+ph(3)*pt(3))
     &            /sqrt(ph(1)**2+ph(2)**2+ph(3)**2)
     &            /sqrt(pt(1)**2+pt(2)**2+pt(3)**2)
      else
       xd(8)=xmin(8)-10.d0
      end if
      if (ib1 > 0) then
       pl=p(:,ilept)
       xd(9)=(ph(1)*pl(1)+ph(2)*pl(2)+ph(3)*pl(3))
     &            /sqrt(ph(1)**2+ph(2)**2+ph(3)**2)
     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
      else
       xd(9)=xmin(9)-10.d0
      end if
      if (ib1 == 0 .and. ib3 > 0 .and. ib4 > 0) then
       xd(10)=sqrt((p(0,ib3)+p(0,ib4))**2-(p(1,ib3)+p(1,ib4))**2
     &            -(p(2,ib3)+p(2,ib4))**2-(p(3,ib3)+p(3,ib4))**2)
      else
       xd(10)=xmin(10)-10.d0
      end if

      do j=1,ndist
       if (nbs(j) > 0) then
        if (xd(j) >= xmin(j) .and. xd(j) <= xmax(j)) then
         do i=1,nbs(j)
          if (xd(j) <= xdi(i,j)) then
           ibin(j)=i
           ncs(i,j)=ncs(i,j)+1
           if (imc > 0) then
            if (imc == j) then
c             if (cross > cdmax(i)) then
             if (cross-cdmax(i) > 1.d-14*cdmax(i)) then
              cdmax(i)=cross
              itmax(i)=iit
              imxdif=1
              imxup=1
             end if
c             call random_number(rand)
             call ranlux(rand,1)
             if (cross > rand*cdmax(i)) then
              ieff=ieff+1
              ncsa(i)=ncsa(i)+1
              if (imxup == 1) then
               close(11)
               write(ind,'(f10.1)') ecm
               open(unit=11,file='mc_events.'//trim(adjustl(ind)),
     &                            status='replace',action='write')
               imxup=0
               ieff=0
              else
               write(11,'(a,i16)') '# Event No. ',ieff
               do k=nisp+1,nepl
                write(11,'(a,i2,a,4(e16.8))') 'p(',k,')=',p(:,k)
               end do
              end if
             end if
            end if
           end if
           exit
          end if
         end do
        else
         ibin(j)=0
         nofr(j)=nofr(j)+1
        end if
       else
        exit
       end if
      end do

      end
